/*
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */
/*
 * HISTORY
 * $Log: GridP.h,v $
 * Revision 1.1.2.2  1994/07/08  21:09:20  daniel
 * 	fix for 9711: grid contraint name
 * 	[1994/07/08  20:40:40  daniel]
 *
 * Revision 1.1.2.1  1994/06/02  15:32:57  marty
 * 	This file has been moved from demos/widgets/Exm/lib. Refer to that directory for prior history.
 * 	[1994/06/01  20:54:13  marty]
 * 
 * Revision 1.1.4.2  1994/03/26  18:23:16  daniel
 * 	add resolve part Index
 * 	[1994/03/26  18:22:05  daniel]
 * 
 * Revision 1.1.4.1  1994/03/26  18:03:02  daniel
 * 	add dialogTitle support
 * 	[1994/03/26  18:02:44  daniel]
 * 
 * Revision 1.1.2.3  1993/10/28  20:32:14  daniel
 * 	add some trait and cleanup
 * 	[1993/10/28  20:31:15  daniel]
 * 
 * Revision 1.1.2.2  1993/10/28  14:31:47  daniel
 * 	fix dynamic layout
 * 	[1993/10/28  14:31:32  daniel]
 * 
 * Revision 1.1.2.1  1993/10/05  21:00:42  yak
 * 	Moved the file from demos/widgets/Exm
 * 	[1993/10/05  17:00:46  yak]
 * 
 * Revision 1.1.4.2  1993/09/22  01:57:52  daniel
 * 	new implementation of the widget
 * 	[1993/09/22  01:55:45  daniel]
 * 
 * 	check point
 * 	[1993/09/10  20:23:26  daniel]
 * 
 * Revision 1.1.2.6  1993/07/21  20:43:02  yak
 * 	Expended OSF Copyright marker
 * 	[1993/07/21  19:19:56  yak]
 * 
 * Revision 1.1.2.5  1993/06/30  21:29:10  rosenberg
 * 	Added comments.
 * 	[1993/06/30  21:28:19  rosenberg]
 * 
 * Revision 1.1.2.4  1993/06/11  17:39:29  rosenberg
 * 	Fixed warning message by defining correct types for
 * 	Layout and CalcSize methods.
 * 	[1993/06/11  17:39:12  rosenberg]
 * 
 * Revision 1.1.2.3  1993/05/24  17:01:51  rosenberg
 * 	Added comments.
 * 	[1993/05/24  17:01:25  rosenberg]
 * 
 * Revision 1.1.2.2  1993/02/26  21:07:19  al
 * 	Initial version
 * 	[1993/02/26  20:52:09  al]
 * 
 * $EndLog$
 */

/*******************************************************************************
 *
 * GridP.h: The widget private header file for the ExmGrid widget.
 *
 ******************************************************************************/

/* Ensure that the file be included only once. */
#ifndef _ExmGridP_h
#define _ExmGridP_h


/* Include appropriate files. */
#include <Exm/Grid.h>  /* public header file for ExmGrid */ 
#include <Xm/ManagerP.h> /* private header file for XmManager */


/* Allow for C++ compilation. */
#ifdef __cplusplus
extern "C" {
#endif


/* Make the following two methods inheritable by subclasses of ExmGrid. */ 
#define ExmInheritLayout   ((ExmLayoutProc) _XtInherit)
#define ExmInheritCalcSize ((ExmCalcSizeProc) _XtInherit)
#define ExmInheritNeedRelayout ((ExmNeedRelayoutProc) _XtInherit)


/* Define new data types for these two inheritable methods. */
#ifdef _NO_PROTO
typedef void (*ExmLayoutProc)();
typedef void (*ExmCalcSizeProc)();
typedef Boolean (*ExmNeedRelayoutProc)();
#else
typedef void (*ExmLayoutProc)(
			Widget,
			Widget);
typedef void (*ExmCalcSizeProc)(
                        Widget,
                        Widget,
                        Dimension *,
                        Dimension *);
typedef Boolean (*ExmNeedRelayoutProc)(
                        Widget,
                        Widget);
#endif

/* Define the widget class part. */
typedef struct
{
	ExmLayoutProc           layout;
        ExmCalcSizeProc         calc_size;
        ExmNeedRelayoutProc     need_relayout;
	XtPointer		extension;
} ExmGridClassPart;


/* Define the full class record. */
typedef struct _ExmGridClassRec
{
	CoreClassPart		core_class;
	CompositeClassPart	composite_class;
	ConstraintClassPart	constraint_class;
	XmManagerClassPart	manager_class;
	ExmGridClassPart	grid_class;
} ExmGridClassRec;

externalref ExmGridClassRec exmGridClassRec;


/* Define the widget instance part. */
typedef struct
{
 /* Provide space for the values of the four resources of ExmGrid. */
	short			rows;
	short			columns;
	Dimension		margin_width;
	Dimension		margin_height;
	XtCallbackList	        map_callback;
	XtCallbackList	        unmap_callback;
	Boolean		        default_position;
	XmRenderTable           button_render_table ;
	XmRenderTable           label_render_table ;
	XmRenderTable           text_render_table ;
        XmString	        dialog_title;

 /* processing_constraints is a flag.  If its value is True, then 
    it means that the ConstraintSetValues method is requesting a 
    geometry change. */
	Boolean			processing_constraints;

} ExmGridPart;

/* Establish an arbitrary limit */
#define EXM_GRID_MAX_NUMBER_OF_ROWS 100
#define EXM_GRID_MAX_NUMBER_OF_COLUMNS 100

/* Define the full instance record. */
typedef struct _ExmGridRec
{
	CorePart		core;
	CompositePart		composite;
	ConstraintPart		constraint;
	XmManagerPart		manager;
	ExmGridPart		grid;
} ExmGridRec;

/* Define the subclassing level index to be used with ResolvePartOffset */
#define ExmGridIndex (XmManagerIndex + 1)

/* Define the constraint part structure. */
typedef struct _ExmGridConstraintPart
{
	Dimension		grid_margin_width_within_cell;
	Dimension		grid_margin_height_within_cell;
} ExmGridConstraintPart, * ExmGridConstraint;


/* Define the full constraint structure. */
typedef struct _ExmGridConstraintRec
{
	XmManagerConstraintPart	manager;
	ExmGridConstraintPart	grid;
} ExmGridConstraintRec, * ExmGridConstraintPtr;


/* Define macros for this class. */
#define ExmGridCPart(w) \
        (&((ExmGridConstraintPtr) (w)->core.constraints)->grid)


/* Allow for C++ compilation. */
#ifdef __cplusplus
} /* Close scope of 'extern "C"' declaration which encloses file. */
#endif


/* Ensure that the file be included only once. */
#endif /* _ExmGridP_h */
