/*
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */
/*
 * HISTORY
 * $Log: SimpleP.h,v $
 * Revision 1.1.2.1  1994/06/02  15:33:12  marty
 * 	This file has been moved from demos/widgets/Exm/lib. Refer to that directory for prior history.
 * 	[1994/06/01  20:54:18  marty]
 *
 * Revision 1.1.4.2  1994/03/26  18:23:19  daniel
 * 	add resolve part Index
 * 	[1994/03/26  18:22:08  daniel]
 * 
 * Revision 1.1.4.1  1994/03/25  19:08:03  daniel
 * 	put support for ContainerItem
 * 	[1994/03/25  19:07:34  daniel]
 * 
 * Revision 1.1.2.1  1993/10/05  21:01:03  yak
 * 	Moved the file from demos/widgets/Exm
 * 	[1993/10/05  17:01:03  yak]
 * 
 * Revision 1.1.2.9  1993/07/21  20:43:25  yak
 * 	Expended OSF Copyright marker
 * 	[1993/07/21  19:21:02  yak]
 * 
 * Revision 1.1.2.8  1993/06/30  20:24:15  rosenberg
 * 	Changed visual_x, visual_y, visual_width, and visual_height to
 * 	one big XRectangle variable named visual.
 * 	[1993/06/30  20:23:26  rosenberg]
 * 
 * Revision 1.1.2.7  1993/06/15  13:33:54  rosenberg
 * 	Changed #include "SimpleP.h" to #include <Exm/SimpleP.h>
 * 	[1993/06/11  19:10:37  rosenberg]
 * 
 * Revision 1.1.2.6  1993/05/26  21:46:05  rosenberg
 * 	Added comments.
 * 	[1993/05/26  21:45:48  rosenberg]
 * 
 * Revision 1.1.2.5  1993/05/24  19:18:32  rosenberg
 * 	Changed 'externref' to 'externalref'
 * 	[1993/05/24  19:18:11  rosenberg]
 * 
 * Revision 1.1.2.4  1993/05/24  16:59:05  rosenberg
 * 	Added comments.
 * 	[1993/05/24  16:58:41  rosenberg]
 * 
 * Revision 1.1.2.3  1993/05/12  20:45:28  rosenberg
 * 	Added comments; changed names of inheritable methods.
 * 	[1993/05/12  20:45:09  rosenberg]
 * 
 * Revision 1.1.2.2  1993/02/26  21:07:46  al
 * 	Initial version
 * 	[1993/02/26  20:52:31  al]
 * 
 * $EndLog$
 */

/*******************************************************************************
 *
 * SimpleP.h: The widget private header file for the ExmSimple demonstration
 *            widget. 
 *
 ******************************************************************************/


/* Ensure that the file be included only once. */
#ifndef _ExmSimpleP_h
#define _ExmSimpleP_h

/* Include appropriate files. */
#include <Exm/Simple.h> /* public header file for the ExmSimple widget */
#include <Xm/PrimitiveP.h> /* private header file for the XmPrimitive widget */

/* Allow for C++ compilation. */
#ifdef __cplusplus
extern "C" {
#endif 

/* Make the following seven methods inheritable by subclasses of ExmSimple. */ 
#define ExmInheritDrawVisual	 ((XtWidgetProc) _XtInherit)
#define ExmInheritDrawShadow	 ((XtWidgetProc) _XtInherit)
#define ExmInheritCreateGC	 ((XtWidgetProc) _XtInherit)
#define ExmInheritDestroyGC	 ((XtWidgetProc) _XtInherit)
#define ExmInheritSelectGC	 ((ExmSelectGCProc) _XtInherit)
#define ExmInheritCalcVisualSize ((XtWidgetProc) _XtInherit)
#define ExmInheritCalcWidgetSize ((XtWidgetProc) _XtInherit)
#define ExmInheritReconfigure    ((ExmReconfigureProc) _XtInherit)

/* Provide typedefs for inheritable methods that do not already
   have an appropriate data type. */
#ifdef _NO_PROTO
typedef GC   (*ExmSelectGCProc)();
typedef void (*ExmReconfigureProc)();
#else
typedef GC   (*ExmSelectGCProc)(
                        Widget);
typedef void (*ExmReconfigureProc)(
			WidgetClass,
			Widget,
			Widget) ;
#endif

/* Define the widget class part. */
typedef struct _ExmSimpleClassPart
{
	XtWidgetProc		draw_visual;
	XtWidgetProc		draw_shadow;
	XtWidgetProc		create_gc;
	XtWidgetProc		destroy_gc;
	ExmSelectGCProc		select_gc;
	XtWidgetProc		calc_visual_size;
	XtWidgetProc		calc_widget_size;
        ExmReconfigureProc      reconfigure; 
	XtPointer		extension;
} ExmSimpleClassPart;

/* Define the full class record. */
typedef struct _ExmSimpleClassRec
{
	CoreClassPart		core_class;
	XmPrimitiveClassPart	primitive_class;
	ExmSimpleClassPart	simple_class;
} ExmSimpleClassRec;

/* Define the subclassing level index to be used with ResolvePartOffset */
#define ExmSimpleIndex (XmPrimitiveIndex + 1)

/* Make the full class record externally accessible. */
externalref ExmSimpleClassRec exmSimpleClassRec;

/* Define the widget instance part. */
typedef struct _ExmSimplePart
{
 /* Provide space for the three resources of ExmSimple. */
	unsigned char		simple_shape;
	Dimension		margin_height;
        Dimension		margin_width;

 /* Provide space for the other protected fields of ExmSimple. */
 /* normal_gc holds a Graphics Context for rendering the visual
    when the value of XmNsensitive is True. */
	GC			normal_gc;

 /* insensitive_gc holds a Graphics Context for rendering the visual
    when the value of XmNsensitive is False. */
	GC			insensitive_gc;

 /* pref_width holds an integral value representing the widget's 
    current preferred width. */
	Dimension		pref_width;

 /* pref_width holds an integral value representing the widget's 
    current preferred height. */
	Dimension		pref_height;

 /* need_to_compute_width is a flag.  If its value is True, then the widget
    needs to renegotiate its width. */
	Boolean			need_to_compute_width;

 /* need_to_compute_height is a flag.  If its value is True, then the widget
    needs to renegotiate its height. */
	Boolean			need_to_compute_height;

 /* visual is an XRectangle value representing the bounding box of the
    widget's visual. */ 
	XRectangle              visual;

 /* need_to_reconfigure is a flag.  If its value is True, then the widget
    needs to call the reconfigure method. */ 
	Boolean			need_to_reconfigure;

 /* saved_foreground is used for the management of the selected state */ 
	Pixel			saved_foreground;
} ExmSimplePart;


/* Define the full instance record. */
typedef struct _ExmSimpleRec
{
	CorePart		core;
	XmPrimitivePart		primitive;
	ExmSimplePart		simple;
} ExmSimpleRec;

/* Allow for C++ compilation. */ 
#ifdef __cplusplus
} /* Close scope of 'extern "C"' declaration which encloses file. */
#endif

/* Ensure that the file be included only once. */
#endif /* _ExmSimpleP_h */
/* Don't add anything after this #endif */
