/* This may look like C code, but it is really -*- C++ -*- */
/*
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */
/*
 * HISTORY
 * $Log: CxxCommand.h,v $
 * Revision 1.1.2.1  1994/06/02  15:33:27  marty
 * 	File moved from demos/widgets/ExmCxx.  Refer to that directory for RCS history.
 * 	[1994/06/01  21:07:27  marty]
 *
 * Revision 1.1.6.1  1994/04/18  21:19:26  drand
 * 	Update C++ widgets and fix some bugs
 * 	[1994/04/18  21:18:42  drand]
 * 
 * Revision 1.1.4.1  1993/10/06  19:01:59  drand
 * 	Wrapped creation routines in extern "C" { ... } and wrapped extern "C" in
 * 	#ifdef __cplusplus.
 * 	[1993/10/06  19:00:18  drand]
 * 
 * Revision 1.1.2.8  1993/07/21  20:44:02  yak
 * 	Expended OSF Copyright marker
 * 	[1993/07/21  19:22:44  yak]
 * 
 * Revision 1.1.2.7  1993/07/01  22:07:35  rosenberg
 * 	Minor tweaks.
 * 	[1993/07/01  22:07:06  rosenberg]
 * 
 * Revision 1.1.2.6  1993/01/27  22:09:02  gilles
 * 	Integrate changes coming from the code review.
 * 	Some issues remain open
 * 	Comments need to be improved
 * 	Should we hide base_class_extension ?
 * 	[1993/01/27  22:06:15  gilles]
 * 
 * Revision 1.1.2.5  1993/01/07  16:58:50  gilles
 * 	Global renaming for C++ :
 * 	        prefix Xm is replaced by XmCxx
 * 	        previous suffixes (Foundry, PP) are removed
 * 	This affects class names, variables, files, directories...
 * 	[1993/01/07  16:52:54  gilles]
 * 
 * Revision 1.1.2.4  1993/01/06  19:16:26  gilles
 * 	Change the management of chained Xt class methods.
 * 	They are now implemented as virtual functions just like the others
 * 	and it is the widget programmer's responsability to chain them.
 * 	[1993/01/06  19:14:44  gilles]
 * 
 * Revision 1.1.2.3  1993/01/03  22:15:17  gilles
 * 	Changing header #define
 * 	[1993/01/03  21:52:47  gilles]
 * 
 * 	Changed name of the widget to CommandPP.
 * 	First version that compiles and links in the source tree.
 * 	[1993/01/03  21:22:45  gilles]
 * 
 * Revision 1.1.2.2  1992/12/18  18:37:50  gilles
 * 	Initial revision.
 * 	[1992/12/18  18:35:11  gilles]
 * 
 * $EndLog$
 */

/*
 * This is the header file for the C++ version of StaticText widget.
 * It contains #ifdef __cplusplus clause to make it possible
 * to include it from a C source file.
 */

#ifndef _CxxCommand_h
#define _CxxCommand_h

#ifdef __cplusplus

#include "CxxString.h"


/**********************************
 * Widget class record definition *
 **********************************/

class ExmCxxCommandClass : public ExmCxxStringClass {

    friend class ExmCxxCommand;

  public:
    
    ExmCxxCommandClass(char*		name,
		      WidgetClass	parent,
		      Cardinal		widgetSize,
		      XtProc		class_init,
		      XtActionList	actions,
		      Cardinal		num_actions,
		      XtResourceList	resources,
		      Cardinal		num_resources,
		      XtInitProc	cxx_cast);
    
    /*
     * static class-methods
     * (must be public : passed as parameter to class_rec static constructor)
     */
    static void		ClassInitialize();

  protected:
    virtual void        class_part_initialize();
};



/***************************
 * Widget class definition *
 ***************************/


struct ExmCxxCommandPart {
  XtCallbackList	activate_callback;

  /* armed is a flag.  If its value is True, then the user has selected the
     widget.  */
  Boolean		armed;

  /* visual_armed is a flag.  If its value is True, then the widget is to
     visually represent its armed state by drawing a certain kind 
     of shadow. */ 
  Boolean		visual_armed;
  
  /* visual_timer holds an integral value representing the time
     (in milliseconds) between when armed is set to True and
     visual_armed is set to True. */ 
  XtIntervalId		visual_timer;
};


class ExmCxxCommand : public ExmCxxString {

    friend class ExmCxxCommandClass;
    friend class ExmCxxCommandActions;
    
  public:
    /*
     * should really be protected but we must reference fields when
     * declaring the static resources array;
     */
    ExmCxxCommandPart	command;
    
  protected:
    
    /*
     * constructors & destructors
     */
    ~ExmCxxCommand();
    
    /*
     * virtual chained class-methods
     */
    // core class part;
    virtual void	initialize(Widget req_w, ArgList, Cardinal*);
    virtual Boolean	set_values(Widget old_w, Widget req_w,
				   ArgList, Cardinal*);
    
    /*
     * virtual self contained class-methods
     */
    // StaticText class part;
    virtual void	draw_shadow();
    
  public:
    
    /*
     * static class-methods
     * (must be public : passed as parameter to class_rec static constructor)
     */
    static void		_MakeCxxWidget(Widget req, Widget new_w,
				       ArgList, Cardinal*);
    /* 
     * Trait methods
     */
    static void		ChangeCB(Widget w, XtCallbackProc activCB,
				 XtPointer closure, Boolean setunset);
    static void		ShowAsDefault(Widget w, XtEnum state);
};

typedef ExmCxxCommandClass	*ExmCxxCommandWidgetClass;
typedef ExmCxxCommand		*ExmCxxCommandWidget;

externalref ExmCxxCommandClass	exmCxxCommandClassRec;

#endif /* __cplusplus */

/* These symbols may be used from C code */
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */
extern Widget			ExmCreateCxxCommand(Widget parent, char *name,
						   ArgList, Cardinal);
#ifdef __cplusplus
}
#endif /* __cplusplus */

externalref WidgetClass		exmCxxCommandWidgetClass;

#ifndef XmIsCxxCommand
#define XmIsCxxCommand(w)	XtIsSubclass(w, exmCxxCommandWidgetClass)
#endif

#endif /* _CxxCommand_h */
/* DON'T ADD ANYTHING AFTER THIS #endif */
