/* 
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */
/*
 * HISTORY
 * $Log: CxxSimpleDemo.cxx,v $
 * Revision 1.1.2.1  1994/06/09  15:55:46  marty
 * 	File name change from .cc suffix to .cxx suffix
 * 	[1994/06/09  15:55:21  marty]
 *
 * Revision 1.1.2.1  1994/06/02  15:33:37  marty
 * 	File moved from demos/widgets/ExmCxx.  Refer to that directory for RCS history.
 * 	[1994/06/01  21:07:25  marty]
 * 
 * Revision 1.1.6.1  1994/04/18  21:19:34  drand
 * 	Update C++ widgets and fix some bugs
 * 	[1994/04/18  21:18:48  drand]
 * 
 * Revision 1.1.4.3  1993/12/10  16:12:59  drk
 * 	CR 7576: Fix compiler warnings.
 * 	[1993/12/10  15:59:37  drk]
 * 
 * Revision 1.1.4.2  1993/09/01  13:34:25  drand
 * 	Changed to use XmCxxString class for XmStrings.
 * 	[1993/09/01  13:33:16  drand]
 * 
 * Revision 1.1.2.7  1993/07/27  21:26:58  drand
 * 	Removed extra comma.
 * 	[1993/07/27  21:10:41  drand]
 * 
 * 	Fixed build problems.
 * 	[1993/07/27  20:58:42  drand]
 * 
 * Revision 1.1.2.5  1993/07/23  21:29:07  drand
 * 	Fixed build problems.
 * 	[1993/07/23  21:28:02  drand]
 * 
 * Revision 1.1.2.4  1993/07/21  20:44:15  yak
 * 	Expended OSF Copyright marker
 * 	[1993/07/21  19:23:23  yak]
 * 
 * Revision 1.1.2.3  1993/07/02  18:44:16  rosenberg
 * 	Corrected comment leader.
 * 	[1993/07/02  18:43:50  rosenberg]
 * 
 * Revision 1.1.2.2  1993/07/01  22:17:36  rosenberg
 * 	Starting version.
 * 	[1993/07/01  22:15:55  rosenberg]
 * 
 * $EndLog$
 */

/****************************************************************************
 ****************************************************************************
 **
 **   File:     simpleDemo.c
 **
 **   Description: C program to instantiate the ExmSimple widget. 
 **
 ****************************************************************************
 ****************************************************************************/


#include <sys/file.h>

/*  Standard C headers  */
#include <stdio.h>
#include <stdlib.h>

/*  X headers  */
#include <X11/IntrinsicP.h>

/*  Xm headers  */
#include <Xm/Xm.h>
#include <Xm/MainW.h>
#include <Xm/Form.h>
#include <Xm/MessageB.h>
#include <Xm/Label.h>
#include <XmCxx/CxxXmString.h>
#include "CxxSimple.h"
#include "CxxString.h"
#include "CxxGrid.h"
#include "CxxCommand.h"

static void TickleCB(Widget, XtPointer, XtPointer);

void TickleCB(
   Widget w,
   XtPointer,
   XtPointer
              )
{
 XmCxxString  stop_that("Stop that!");
 Arg       arg[2];
 Widget    MsgBox1;
 printf("In TickleCB\n");

 XtSetArg(arg[0], XmNmessageString, (XmString) stop_that);
 
 MsgBox1 = XmCreateMessageDialog(w, "MsgBox1",  arg, 1);

 XtManageChild(MsgBox1);
} 


void make_simple_widgets(Widget parent)
{
  Widget       simpleWidget1, simpleWidget2, stringWidget,
    commandButtonWidget1;
  XmCxxString  compound_string1, compound_string2;
  Arg          args[8];
  Cardinal     argcount;

  simpleWidget1 = XtCreateManagedWidget("Simple1", 
					exmCxxSimpleWidgetClass, parent, 
					NULL, 0);

  argcount = 0;
  XtSetArg(args[argcount], ExmNsimpleShape, ExmSHAPE_RECTANGLE); argcount++;
  XtSetArg(args[argcount], XmNmarginWidth, 20); argcount++;

  simpleWidget2 = XtCreateManagedWidget("Simple2", 
					exmCxxSimpleWidgetClass, parent, 
					args, argcount);

  compound_string1 = "ExmString";
  compound_string1 = compound_string1 + XmCxxString::Separator() + "widget";

  argcount = 0;
  XtSetArg(args[argcount], ExmNcompoundString, (XmString) compound_string1);
  argcount++;
  stringWidget = XtCreateManagedWidget("String", 
				       exmCxxStringWidgetClass, parent, 
				       args, argcount);

  compound_string2 = "ExmCommandButton";
  compound_string2 = compound_string2 + XmCxxString::Separator() + "Push me";

  argcount = 0;
  XtSetArg(args[argcount], ExmNcompoundString, (XmString) compound_string2);
  argcount++;
  commandButtonWidget1 = XtCreateManagedWidget("CommandButtonWidget1", 
					       exmCxxCommandWidgetClass, 
					       parent, args, argcount);

  XtAddCallback(commandButtonWidget1, XmNactivateCallback, TickleCB, NULL);
}

#define      APP_CLASS  "C++ Widgets demo"

int main (int argc, char **argv)
{
  Display     *display;
  Widget       shell, mainWindow, workRegion;
  XtAppContext AppContext;
  Arg args[8];
  Cardinal argcount = 0;


  XtToolkitInitialize();
  AppContext = XtCreateApplicationContext();
  if ((display = XtOpenDisplay (AppContext, NULL, argv[0], APP_CLASS,
				NULL, 0, &argc, argv)) == NULL)
    {
      fprintf (stderr,"\n%s:  Can't open display\n", argv[0]);
      exit(1);
    }

  argcount = 0;
  XtSetArg(args[argcount], XmNallowShellResize, True); argcount++;

  shell = XtAppCreateShell(argv[0], APP_CLASS, applicationShellWidgetClass,
			   display, args, argcount);
			     

  mainWindow = XtCreateManagedWidget("myMainWindow", xmMainWindowWidgetClass, 
				     shell, NULL, 0);

  argcount = 0;
  XtSetArg(args[argcount], XmNrows, 2); argcount++;
  XtSetArg(args[argcount], XmNcolumns, 2); argcount++;
  workRegion = XtCreateManagedWidget("myWorkRegion", exmCxxGridWidgetClass,
				     mainWindow, args, argcount); 

  make_simple_widgets(workRegion);

  XmMainWindowSetAreas(mainWindow, NULL, NULL, NULL, NULL, workRegion);

  XtRealizeWidget(shell);

  XtAppMainLoop(AppContext);
}
