/*
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */
/*
 * HISTORY
 * $Log: CxxString.h,v $
 * Revision 1.1.2.1  1994/06/02  15:33:40  marty
 * 	File moved from demos/widgets/ExmCxx.  Refer to that directory for RCS history.
 * 	[1994/06/01  21:07:30  marty]
 *
 * Revision 1.1.6.1  1994/04/18  21:19:38  drand
 * 	Update C++ widgets and fix some bugs
 * 	[1994/04/18  21:18:51  drand]
 * 
 * Revision 1.1.4.1  1993/10/06  19:02:10  drand
 * 	Wrapped creation routines in extern "C" { ... } and wrapped extern "C" in
 * 	#ifdef __cplusplus.
 * 	[1993/10/06  19:01:01  drand]
 * 
 * Revision 1.1.2.3  1993/07/21  20:44:19  yak
 * 	Expended OSF Copyright marker
 * 	[1993/07/21  19:23:33  yak]
 * 
 * Revision 1.1.2.2  1993/07/01  22:13:34  rosenberg
 * 	Not in synch with Exm/StringP.h
 * 	[1993/07/01  22:13:04  rosenberg]
 * 
 * $EndLog$
 */

/*
 * CxxString.h - Class definitions for CxxString
 */

#ifndef _ExmCxxString_h
#define _ExmCxxString_h

#ifdef __cplusplus

#include "./CxxSimple.h"
#include <Xm/AccTextT.h> /* header file for AccessTextual trait */


/**********************************
 * Widget class record definition *
 **********************************/

class ExmCxxStringClass : public ExmCxxSimpleClass {
  friend class ExmCxxString;
public:
  ExmCxxStringClass( char*		name,
		    WidgetClass	        parent,
		    Cardinal		widgetSize,
		    XtProc		class_init,
		    XtActionList	actions,
		    Cardinal		num_actions,
		    XtResourceList	resources,
		    Cardinal		num_resources,
		    XtInitProc		cxx_cast);
    
 // public static methods
  static  void      ClassInitialize();

protected:
  virtual void      class_part_initialize(); 
  unsigned char     default_render_table_type;
};

/* Define a new enumerated type. */
typedef enum { ExmCENTER_STRING, ExmSTART_STRING_LEFT_SIDE,
                     ExmSTART_STRING_RIGHT_SIDE} ExmHowAligned;


/* Define string equivalents of new resource names. */
#define ExmNcompoundString "compoundString"
#define ExmCCompoundString "CompoundString"


typedef struct _exmCxxStringPart
{
 /* Provide space for the five resources of ExmString. */
	XmString		compound_string;
	unsigned char		alignment;
        XmRenderTable           render_table;
        Boolean                 recompute_size;

        ExmHowAligned           text_starts_here; 
} ExmCxxStringPart;


class ExmCxxString : public ExmCxxSimple {
  friend class ExmCxxStringClass;
  friend class ExmCxxStringActions;
public:
  ExmCxxStringPart string;

  ~ExmCxxString();

  // core class part
  virtual void	    initialize(Widget req_w, ArgList, Cardinal*);
  virtual Boolean   set_values(Widget old_w, Widget req_w,
                               ArgList, Cardinal*);
  virtual void resize();
  virtual XtGeometryResult query_geometry(XtWidgetGeometry*, XtWidgetGeometry*);

  // override certain methods of primitive class extension record.
  virtual Boolean             widget_baseline(Dimension** dim, int* nb);
  virtual Boolean             widget_display_rect(XRectangle*);
    
  // override certain methods of simple class 
  virtual void draw_visual(void);
  virtual void create_gc(void);
  virtual void calc_visual_size(void);
  
  // provide methods for string class 
  virtual void AlignmentDirection(void);

  static void DefaultFont (Widget, int, XrmValue*);
  static void GetValuesCompoundString(Widget, int, XtArgVal *);

  static XtPointer StringGetValue(Widget, int);
  static void StringSetValue(Widget, XtPointer, int);
  static int StringPreferredFormat(Widget);

 public:
  // static class-methods: passed as parameter to class_rec static constructor.
  static void _MakeCxxWidget(Widget, Widget new_w, ArgList, Cardinal*);
};

typedef class ExmCxxStringClass     *ExmCxxStringWidgetClass;
typedef class ExmCxxString          *ExmCxxStringWidget;

externalref   ExmCxxStringClass      exmCxxStringClassRec;
#endif /* __cplusplus */

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */
extern Widget ExmCreateCxxString(Widget, char*, ArgList, Cardinal);
#ifdef __cplusplus
}
#endif /* __cplusplus */

externalref WidgetClass exmCxxStringWidgetClass;

#endif /* _ExmCxxString_h */
