/*
 * COPYRIGHT NOTICE
 * Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (MOTIF).  See the file named COPYRIGHT.MOTIF
 * for the full copyright text.
 * 
 */
/*
 * HISTORY
 * $Log: free.c,v $
 * Revision 1.1.2.1  1994/06/02  15:33:48  marty
 * 	File moved from demos/workspace/wsmLib
 * 	[1994/06/01  21:08:48  marty]
 *
 * 	This file has been moved from demos/workspace/wsmlib.  Refer to that directory for prior history
 *
 * Revision 1.1.2.1  1994/03/23  17:59:09  deblois
 * 	Split wsm library code into mwm and wsm dirs.
 * 	[1994/03/23  17:57:21  deblois]
 * 
 * $EndLog$
 */

#include "wsm_proto.h"
#include <X11/Xatom.h>

static void FreeWinData(
#ifndef _NO_PROTO
WSMWinData *data, int num
#endif
);

/*	Function Name: FreeRequest
 *	Description: Frees data in the request that has been allocated.
 *	Arguments: request - the request.
 *	Returns: none.
 */

void
#ifndef _NO_PROTO
FreeRequest(WSMRequest *request)
#else
FreeRequest(request)
WSMRequest *request;
#endif
{
    register int i;

    if (!request->any.allocated)
	return;			/* Nothing to free. */

    switch(request->any.type) {
    case WSM_CONNECT:
	XtFree((XtPointer) request->connect.known_versions);
	break;
    case WSM_EXTENSIONS:
        {
	    register int num = request->extensions.num_extensions;
	    register String *ptr;

	    num = request->extensions.num_extensions;
	    ptr = request->extensions.extension_suggestions;
	    for (i = 0; i < num; i++, ptr++) 
		XtFree((XtPointer) *ptr);

	    XtFree((XtPointer) request->extensions.extension_suggestions);
	}
	break;
    case WSM_SET_STATE:
        {
	    register int num = request->set_state.num_win_info_list;
	    WSMWinInfo *win_info = request->set_state.win_info_list;

	    for (i = 0; i < num; i++, win_info++) 
		FreeWinData(win_info->data_list, win_info->num_data_list);

	    XtFree((XtPointer) request->set_state.win_info_list);
	}
	break;
    default:
	break;
    }
}

/*	Function Name: FreeRequest
 *	Description: Frees data in the request that has been allocated.
 *	Arguments: reply - the reply.
 *	Returns: none.
 */

void
#ifndef _NO_PROTO
FreeReply(WSMReply *reply)
#else
FreeReply(reply)
WSMReply *reply;
#endif
{
    register int i;

    if (!reply->any.allocated)
	return;			/* Nothing to free. */

    switch(reply->any.type) {
    case WSM_EXTENSIONS:
        {
	    register int num = reply->extensions.num_extensions;
	    register String *ptr;

	    num = reply->extensions.num_extensions;
	    ptr = reply->extensions.extensions;
	    for (i = 0; i < num; i++, ptr++) 
		XtFree((XtPointer) *ptr);

	    XtFree((XtPointer) reply->extensions.extensions);
	}
	break;
    case WSM_GET_STATE:
        {
	    WSMWinInfo *win_info = reply->get_state.win_info_list;
	    register int num = reply->get_state.num_win_info_list;

	    for (i = 0; i < num; i++, win_info++) 
		FreeWinData(win_info->data_list, win_info->num_data_list);

	    XtFree((XtPointer) reply->get_state.win_info_list);
	}
	break;
    case WSM_REG_WINDOW:
	FreeWinData(reply->register_window.window_data,
		    reply->register_window.num_window_data);
	break;
    default:
	break;
    }
}

/************************************************************
 *
 *  Internal routines
 *
 ************************************************************/

/*	Function Name: FreeWinData
 *	Description: Frees the data associated with this window.
 *	Arguments: data - The data to free.
 *                 num - number of elements in data.
 *	Returns: none.
 */

static void
#ifndef _NO_PROTO
FreeWinData(WSMWinData *data, int num)
#else
FreeWinData(data, num)
WSMWinData *data;
int num;
#endif
{
    register int i;
    WSMWinData *top = data;

    for (i = 0; i < num; i++, data++) {
	switch (data->type) {
	case WSM_CHAR_LIST_DATA:
	    XtFree((XtPointer) data->data.char_ptr);
	    break;
	case WSM_SHORT_LIST_DATA: 
	    XtFree((XtPointer) data->data.short_ptr);
	    break;
	case WSM_LONG_LIST_DATA:
	    XtFree((XtPointer) data->data.long_ptr);
	    break;
	default:
	    break;
	}
    }
    
    XtFree((XtPointer) top);
}	
