/*
 * COPYRIGHT NOTICE
 * Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (MOTIF).  See the file named COPYRIGHT.MOTIF
 * for the full copyright text.
 * 
 */
/*
 * HISTORY
 * $Log: AdjView.c,v $
 * Revision 1.1.2.1  1994/06/30  17:50:55  daniel
 * 	cr 9583: demo compliance
 * 	[1994/06/30  17:50:26  daniel]
 *
 * $EndLog$
 */
/*******************************************************************
   The following code is aimed at constraining the work_window size
   in an AUTOMATIC scrolledwindow so that it never becomes smaller
   than the view, in other word, so that you can never see the clipwindow
   background.
   First there are two callback/handler routines and then the convenience
   function that installs them on the clip and work window.
*******************************************************************/

#include <Xm/XmAll.h>

#ifdef _NO_PROTO

static void ClipResize ();
static void WorkResize();

#else
static void ClipResize (Widget clip_window, 
			XtPointer client_data, XtPointer call_data) ;
static void WorkResize(Widget work_window, 
		       XtPointer client_data, XEvent *event, Boolean *cont);
#endif



/* The following hack needs more thinking, it appears as if the scrollbar 
   highlight thickness must be removed from the work_window new size.
   If that is the model, then the real vertical or scrollbar ht values would
   have to be used instead of this constant */
#define SB_HT 2

static void 
#ifdef _NO_PROTO
ClipResize (clip_window, client_data, call_data) 
Widget		clip_window;		/*  widget id		*/
XtPointer       client_data;	/*  data from application   */
XtPointer       call_data;	/*  data from widget class  */
#else
ClipResize (Widget clip_window, XtPointer client_data, XtPointer call_data) 
#endif
{
    Widget work_window = (Widget) client_data ;
    static Dimension prev_clip_width = 0, prev_clip_height = 0 ;
    Dimension clip_width, clip_height, work_width, work_height ;

    /* if we're getting bigger, check the current work_window size
       and if we're bigger than it, resize the work_window so it just
       fits the clipwindow.
       if we're getting smaller, ask the preferred work_window size,
       and if we're still bigger than it, resize the work_window so it just
       fits the clipwindow, else resize it to its preferred size. */

    XtVaGetValues(clip_window, XmNwidth, &clip_width, 
		  XmNheight, &clip_height, NULL);

    if (clip_width > prev_clip_width) {
	XtVaGetValues(work_window, XmNwidth, &work_width, NULL);
	if (clip_width > work_width) {
	    XtVaSetValues(work_window, XmNwidth, clip_width - SB_HT, NULL);
	}
    } else {
	XtWidgetGeometry  preferred;
	XtQueryGeometry(work_window, NULL, &preferred);
	if (clip_width > preferred.width) {
	    XtVaSetValues(work_window, XmNwidth, clip_width - SB_HT, NULL);
	} else {
	    XtVaSetValues(work_window, XmNwidth, preferred.width, NULL);
	}
    }    
    prev_clip_width = clip_width ;

    if (clip_height > prev_clip_height) {
	XtVaGetValues(work_window, XmNheight, &work_height, NULL);
	if (clip_height > work_height) {
	    XtVaSetValues(work_window, XmNheight, clip_height - SB_HT, NULL);
	}
    } else {
	XtWidgetGeometry  preferred;
	XtQueryGeometry(work_window, NULL, &preferred);
	if (clip_height > preferred.height) {
	    XtVaSetValues(work_window, XmNheight, clip_height - SB_HT, NULL);
	} else {
	    XtVaSetValues(work_window, XmNheight, preferred.height, NULL);
	}
    }    
    prev_clip_height = clip_height ;
}

static void 
#ifdef _NO_PROTO
WorkResize(work_window, client_data, event, cont)
Widget work_window;
XtPointer client_data ;
XEvent * event ;
Boolean *cont ;
#else
WorkResize(Widget work_window, XtPointer client_data, XEvent *event, Boolean *cont)
#endif
{
    Widget clip_window = (Widget) client_data ;
    Dimension clip_width, work_width, clip_height, work_height ;

    /* if the work_window is getting smaller than its clip_window
       parent, resize it so it just fits. */
    XtVaGetValues(work_window, XmNwidth, &work_width, 
		  XmNheight, &work_height, NULL);
    XtVaGetValues(clip_window, XmNwidth, &clip_width, 
		  XmNheight, &clip_height, NULL);
    if (clip_width > work_width + 1) {
	XtVaSetValues(work_window, XmNwidth, clip_width - SB_HT, NULL);
    }
    if (clip_height > work_height + 1) {
	XtVaSetValues(work_window, XmNheight, clip_height - SB_HT, NULL);
    }
}

/*---------That's the public API ----------------------*/
void 
#ifdef _NO_PROTO
XmdSWAdjustView (sw) 
Widget		sw ;
#else
XmdSWAdjustView (Widget sw)
#endif
{
    Widget clip_window, work_window ;
    unsigned char scrolling_policy ;

    /* get the clipwindow and _a_ workwindow.
       To be really correct, this code would have to figure out 
       the case where there are more than one work window */
    XtVaGetValues(sw, XmNclipWindow, &clip_window, XmNworkWindow, 
		  &work_window, XmNscrollingPolicy, &scrolling_policy, NULL);

    if (scrolling_policy != XmAUTOMATIC) return ;

     /* attach callback to ClipWindow so that Container never
       goes smaller than clip window. We know ClipWindow is
       a DrawingArea subclass, so we can use a resizeCallback */
    XtAddCallback(clip_window, XmNresizeCallback, ClipResize, 
		  (XtPointer) work_window);
    /* also attach handler to Container sp that it never
       goes smaller than clip window by itself */
    XtAddEventHandler(work_window, StructureNotifyMask, False, WorkResize, 
		      (XtPointer) clip_window);
}

