/*
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */
/*
 * HISTORY
 * $Log: airport.h,v $
 * Revision 1.1.2.1  1994/06/02  15:34:42  marty
 * 	File moved from ./demos directory.
 * 	[1994/06/02  14:21:29  marty]
 *
 * Revision 1.1.2.3  1993/12/08  23:42:03  yak
 * 	Expended copyright. CR 7534
 * 	[1993/12/08  23:01:46  yak]
 * 
 * Revision 1.1.2.2  1993/08/26  21:52:29  vania
 * 	Initial revision
 * 	[1993/08/26  21:48:28  vania]
 * 
 * 	created file
 * 	[1993/08/26  21:21:11  vania]
 * 
 * $EndLog$
 */

/*
 * The Airport Drag And Drop Demo
 *
 *    Airport Data Structure
 *
 *
 *    Created: Vania JOLOBOFF / May 1992
 *
 *
 * ======================================================================
 * 
 */

/*
 * ===================== include ========================================
 */

#if (! defined airport_h)
#define airport_h 

#include <stdlib.h>
#include <stdio.h>
#include <Xm/XmAll.h>

/* ======================================================================
 *  General Types
 */

typedef struct _Source {
   Widget departure;
   Widget over_ocean;
   Widget over_land;
   Widget over_site;
   Widget landing;
   Widget landed;
} SourceRec, *Source;

typedef struct _Destination {
   Widget enter;
   Widget leave;
   Widget landing;
   Widget parked;
   int gate;
} DestinationRec, *Destination;

typedef struct _Spot {
   Boolean empty;
   Position x,y;
} SpotRec, *Spot;

typedef enum {empty, normal, full} ParkState ;
 
typedef struct _Park {
   ParkState state;
   GC gc;
   int spot_count;
   int plane_count;
   Spot spots; /* variable length array of spots */
} ParkRec, *Park;

typedef struct _Track {
   int borderwidth;
   XRectangle bbox;
   GC gc;
} TrackRec, *Track;

typedef enum {closed, open} AirportState ;

/*
 * The airport data structure consists of
 * 	- the widget needed in various callbacks 
 *	- the drag source data
 *	- the drop site data
 *	- the details of the track graphics 
 */

typedef struct _Airport {
   XtAppContext context; 
   Display * display;
   Window ocean;
   int screen_width;
   int screen_height;
   AirportState state;
   Widget main; /* the main window */
   Widget airport; /* the airport drawing area */
   Widget help_dialog;
   Widget error_dialog;
   Widget warning_dialog;
   Widget dragIcon;
   Widget msg_area;
   SourceRec from; /* the drag data */
   DestinationRec to; /* drop site data */
   ParkRec park; /* info on available gates at the airport */
   TrackRec track; /* info on track graphics */
} AirportRec, *Airport;

/*
 * A flight has a flight number. It starts from some airport (from)
 * When landing, it will be taxi-ed to some gate.
 */

typedef struct _Flight {
   long number;
   int gate;     /* the gate it's going to be taxi-ed */
   Airport from; /* the airport it took off */
} FlightRec, *Flight;

/* ======================================================================
 * Global constants
 */

#define plane_width 25
#define plane_height 25

/* ======================================================================
 * Global variables 
 */

#ifdef DECLAREGLOBAL
#define global
#else
#define global extern
#endif

global Atom FlightAtom;
global Atom DeleteAtom;

/* 
 * Application Resources
 */

global struct _AirportResources {
   String too_small;
   Pixel spot_background;
   Pixel spot_foreground;
   Pixel flight_background;
   Pixel flight_foreground;
   Pixel invalid_foreground;
   Pixel valid_foreground;
   Pixel none_foreground;
   Pixel track_background;
   Pixel track_foreground;
   Pixel drag_under_background;
   Pixel drag_under_foreground;
   Dimension track_border_width;
   Dimension track_margin;
   int bell_duration;
   int bell_percent;
} AirportResources;

#ifdef _NO_PROTO
global void DrawAirport();
global void Paging();
#else
global void DrawAirport(Airport this, Window wd, Pixel bg, Pixel fg);
/* draw the tracks and parked planes */

global void Paging(Airport this, String msg);
/* display a message in message area */

#endif

#endif /* airport_h */
