/*
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */
/*
 * HISTORY
 * $Log: dragsource.c,v $
 * Revision 1.1.2.1  1994/06/02  15:34:43  marty
 * 	File moved from ./demos directory.
 * 	[1994/06/02  14:21:34  marty]
 *
 * Revision 1.1.2.6  1993/12/08  23:42:04  yak
 * 	Expended copyright. CR 7534
 * 	[1993/12/08  23:01:48  yak]
 * 
 * Revision 1.1.2.5  1993/09/14  21:15:31  vania
 * 	*** empty log message ***
 * 	[1993/09/14  21:14:35  vania]
 * 
 * Revision 1.1.2.4  1993/09/13  12:09:27  vania
 * 	*** empty log message ***
 * 	[1993/09/13  12:08:18  vania]
 * 
 * Revision 1.1.2.3  1993/09/09  15:58:56  vania
 * 	add no proto
 * 	[1993/09/09  15:57:38  vania]
 * 
 * Revision 1.1.2.2  1993/08/26  21:52:34  vania
 * 	Initial revision
 * 	[1993/08/26  21:48:33  vania]
 * 
 * 	*** empty log message ***
 * 	[1993/08/26  21:23:58  vania]
 * 
 * $EndLog$
 */

/*
 * The Airport Drag And Drop Demo
 *
 * The module that contains all drag source / initiator code
 *
 *    Created: Vania JOLOBOFF / May 1992
 *
 */

#include <Xm/XmAll.h>
#include "airport.h"
#include "dragsource.h"

/* ============================ prototypes  ======================= */

#ifdef _NO_PROTO
static void EnterTopLevel();
static void LeaveTopLevel();
static void DropSiteEnter();
static void DropSiteLeave();
static void Flying();
static void OperationChanged();
static void PlaneLanding();
static void PlaneHasLanded();
static void FlightFinished();
static Boolean SendPlaneID();
#else
static void EnterTopLevel(Widget dc, Airport this,
			  XmTopLevelEnterCallbackStruct *call_data);
/* drag callback entering top level window */

static void LeaveTopLevel(Widget dc, Airport this,
			  XmTopLevelLeaveCallbackStruct *call_data);
/* drag callback leaving top level window */

static void DropSiteEnter(Widget dc, Airport this,
			  XmDropSiteEnterCallbackStruct *call_data);
/* drag callback entering drop site */

static void DropSiteLeave(Widget dc, Airport this,
			  XmDropSiteLeaveCallbackStruct *call_data);
/* drag callback leaving drop site */

static void Flying(Widget dc, Airport this,
		   XmDragMotionCallbackStruct *call_data);
/* drag motion callback on initiator */

static void OperationChanged(Widget dc, Flight flight,
			     XmOperationChangedCallbackStruct *call_data);
/* side effect change callback on initiator */

static void PlaneLanding(Widget dc, Flight flight,
			 XmDropStartCallbackStruct *call_data);
/* drop start callback on initiator side */

static void PlaneHasLanded(Widget dc, Flight flight,
			   XmDropFinishCallbackStruct *call_data);
/* drop end callback on initiator side */

static void FlightFinished(Widget dc, Flight flight,
				  XmDragDropFinishCallback *call_data);
/* drag drop finish callback on initiator. Remove data for that plane */

static Boolean SendPlaneID(Widget drag_context, Atom *selection, Atom *target, 
			 Atom *type,
			 XtPointer *value, unsigned long *length,
			 int *format); 

/* The Xt selection convert proc sending the data to the drop site */

#endif

/* ===============================================================
 *  Start dragging a plane icon.
 */

#ifdef _NO_PROTO
void TakeOff(area, this, evt, spot)
		Widget area;
		Airport this;
		XButtonEvent *evt;
		int spot;
#else
void TakeOff(Widget area, Airport this, XButtonEvent *evt, int spot)
#endif
{
   Arg args[20];
   int n;
   Flight flight;
   Widget dc;
   char msg[256];
   static Boolean init = True;

   if (init) {
      srand((unsigned int) XtWindow(area));
      init = False;
   }

   flight = (Flight)XtCalloc(sizeof(FlightRec), 1);
   flight->number = rand();
   flight->from = this;
   flight->gate = spot;
   sprintf(msg, "Flight %d leaving from gate %d",
	   flight->number, flight->gate);
   Paging(this, msg);
   n = 0;
   XtSetArg(args[n], XmNclientData, flight); n++;
   XtSetArg(args[n], XmNconvertProc, SendPlaneID); n++;
   XtSetArg(args[n], XmNcursorBackground,
	    AirportResources.flight_background); n++;
   XtSetArg(args[n], XmNcursorForeground,
	    AirportResources.flight_foreground); n++;
   XtSetArg(args[n], XmNvalidCursorForeground,
	    AirportResources.valid_foreground); n++;
   XtSetArg(args[n], XmNinvalidCursorForeground,
	    AirportResources.invalid_foreground); n++;
   XtSetArg(args[n], XmNnoneCursorForeground,
	    AirportResources.none_foreground); n++ ;
   XtSetArg(args[n], XmNdragOperations, XmDROP_MOVE) ; n++;
   XtSetArg(args[n], XmNblendModel, XmBLEND_JUST_SOURCE) ; n++;
   XtSetArg(args[n], XmNsourceCursorIcon, this->dragIcon) ; n++;
   XtSetArg(args[n], XmNexportTargets, &FlightAtom); n++;
   XtSetArg(args[n], XmNnumExportTargets, 1); n++;
   dc = XmDragStart(area, (XEvent *)evt, args, n);
   XtAddCallback(dc, XmNdragMotionCallback,
		 (XtCallbackProc) Flying, (XtPointer) this);
   XtAddCallback(dc, XmNdropSiteEnterCallback,
		 (XtCallbackProc) DropSiteEnter, (XtPointer) this);
   XtAddCallback(dc, XmNdropSiteLeaveCallback,
		 (XtCallbackProc) DropSiteLeave, (XtPointer) this);
   XtAddCallback(dc, XmNtopLevelEnterCallback,
		 (XtCallbackProc) EnterTopLevel, (XtPointer) this);
   XtAddCallback(dc, XmNtopLevelLeaveCallback,
		 (XtCallbackProc) LeaveTopLevel, (XtPointer) this);
   XtAddCallback(dc, XmNdropStartCallback,
		 (XtCallbackProc) PlaneLanding, (XtPointer) flight);
   XtAddCallback(dc, XmNdropFinishCallback,
		 (XtCallbackProc) PlaneHasLanded, (XtPointer) flight);
   XtAddCallback(dc, XmNdragDropFinishCallback,
		 (XtCallbackProc) FlightFinished , (XtPointer) flight);
   XtAddCallback(dc, XmNoperationChangedCallback,
		 (XtCallbackProc) OperationChanged, (XtPointer) flight);

}

/* ===============================================================
 * Pointer enters a new top level window.
 * Check if this is the ocean or new land
 */

#ifdef _NO_PROTO
static void EnterTopLevel(dc, this, call_data)
		Widget dc;
		Airport this;
		XmTopLevelEnterCallbackStruct *call_data;
#else
static void EnterTopLevel(Widget dc, Airport this,
			  XmTopLevelEnterCallbackStruct *call_data)
#endif
{

   if (call_data->window != this->ocean)
     XmToggleButtonSetState(this->from.over_land, True, True);
}

/* ===============================================================
 *  Pointer leaves current top level window.
 *  Assume we are back on the ocean. Next enter will reset the right value.
 */

#ifdef _NO_PROTO
static void LeaveTopLevel(dc, this, call_data)
		Widget dc;
		Airport this;
		XmTopLevelLeaveCallbackStruct *call_data;
#else
static void LeaveTopLevel(Widget dc, Airport this,
			  XmTopLevelLeaveCallbackStruct *call_data)
#endif
{  
   XmToggleButtonSetState(this->from.over_ocean, True, True);
}

/* ===============================================================
 * Someone is entering over the airport.
 * Signal radar detection.
 */

#ifdef _NO_PROTO
static void DropSiteEnter(dc, this, call_data)
		Widget dc;
		Airport this;
		XmDropSiteEnterCallbackStruct *call_data;
#else
static void DropSiteEnter(Widget dc, Airport this,
			  XmDropSiteEnterCallbackStruct *call_data)
#endif
{
   XmToggleButtonSetState(this->from.over_site, True, True);
}

/* ===============================================================
 * The thing that was over is leaving the airport.
 * Radar contact lost.
 */

#ifdef _NO_PROTO
static void DropSiteLeave(dc, this, call_data)
		Widget dc; 
		Airport this;
		XmDropSiteLeaveCallbackStruct *call_data;
#else
static void DropSiteLeave(Widget dc, Airport this,
			  XmDropSiteLeaveCallbackStruct *call_data)
#endif
{  
   XmToggleButtonSetState(this->from.over_land, True, True);
}

/* ===============================================================
 * Motion callback on source side.
 * This is where we would could do fancy animation of the source icon.
 * Here we just beep.
 */

#ifdef _NO_PROTO
static void Flying(dc, this, call_data)
		Widget dc;
		Airport this;
		XmDragMotionCallbackStruct *call_data;
#else
static void Flying(Widget dc,  Airport this,
		   XmDragMotionCallbackStruct *call_data)
#endif
{
   static Boolean first = True;

   if (first) 
     { 
	XKeyboardControl controlValues;
	unsigned long valueMask = KBBellPercent | KBBellPitch | KBBellDuration;

	controlValues.bell_percent  = AirportResources.bell_percent;
	controlValues.bell_pitch    = 440; 
	controlValues.bell_duration = AirportResources.bell_duration;
	XChangeKeyboardControl(this->display, valueMask, &controlValues);
	first = False; 
     }

   XBell(this->display, 100);

}

/* ===============================================================
 * The drop start callback on the source side.
 * Set state to plane landing
 */

#ifdef _NO_PROTO
static void PlaneLanding(dc, flight, call_data)
		Widget dc;
		Flight flight;
		XmDropStartCallbackStruct *call_data;
#else
static void PlaneLanding(Widget dc, Flight flight,
			 XmDropStartCallbackStruct *call_data)
#endif
{
   XmToggleButtonSetState(flight->from->from.landing, True, True);
}


/* ===============================================================
 * The operation changed callback. Planes only move, no copy
 */

#ifdef _NO_PROTO
static void OperationChanged(dc, flight, call_data)
		Widget dc;
		Flight flight;
		XmOperationChangedCallbackStruct *call_data;
#else
static void OperationChanged(Widget dc, Flight flight,
			   XmOperationChangedCallbackStruct *call_data)
#endif
{

}

/* ===============================================================
 * The drop finish callback. The plane has landed somewhere
 *  Hold on, it's not finished yet
 */

#ifdef _NO_PROTO
static void PlaneHasLanded(dc, flight, call_data)
		Widget dc;
		Flight flight;
		XmDropFinishCallbackStruct *call_data;
#else
static void PlaneHasLanded(Widget dc, Flight flight,
			   XmDropFinishCallbackStruct *call_data)
#endif
{
   Airport this = flight->from;
   char msg[256];

   XmToggleButtonSetState(this->from.landed, True, True);

   if (call_data->completionStatus == XmDROP_SUCCESS) {
      Spot spot = &(this->park.spots[flight->gate]);
      
      spot->empty = True;
      XClearArea(this->display, XtWindow(this->airport), 
		 spot->x, spot->y, plane_width, plane_height, False);
     sprintf(msg, "Passengers from flight %d have landed safe",
	      flight->number);
   }
   else
     sprintf(msg, "Flight %d has returned", flight->number);
   Paging(this, msg);
}

/* ===============================================================
 * the drag drop finish callback. Clean data structures.
 */

#ifdef _NO_PROTO
static void FlightFinished(dc, flight, call_data)
		Widget dc;
		Flight flight;
		XmDragDropFinishCallback *call_data;
#else
static void FlightFinished(Widget dc, Flight flight,
				  XmDragDropFinishCallback *call_data)
#endif
{
   Airport this = flight->from;

   XmToggleButtonSetState(this->from.departure, True, True);
   XtFree((char *) flight);
}
/* ===============================================================
 *  Xt convert selection proc. 
 *  Send flight ID on drop transfer request.
 *  Delete the source plane image on Delete. You know it's arrived there.
 */

#ifdef _NO_PROTO
static Boolean SendPlaneID(drag_context, selection, target, type,
			   value, length, format)
		Widget drag_context; 
		Atom *selection; 
		Atom *target; 
		Atom *type;
		XtPointer *value;
		unsigned long *length;
		int *format;
#else
static Boolean SendPlaneID(Widget drag_context, Atom *selection, Atom *target, 
			 Atom *type,
			 XtPointer *value, unsigned long *length,
			 int *format)
#endif
{
   Airport this;
   Flight flight;

   XtVaGetValues(drag_context, XmNclientData, &flight, NULL);
   this = flight->from;

   if (*target == FlightAtom) {
      long * data = (long *) XtMalloc(sizeof(long));

      *data = flight->number;
      *type = FlightAtom;
      *value = (XtPointer) data;
      *format = 32;
      *length = 1;
      return True;
   }
   else {
      Paging(flight->from, "Erroneous messsage on radio");
      return False;
   }
}
