/*
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */
/*
 * HISTORY
 * $Log: dropsite.c,v $
 * Revision 1.1.2.1  1994/06/02  15:34:46  marty
 * 	File moved from ./demos directory.
 * 	[1994/06/02  14:21:43  marty]
 *
 * Revision 1.1.2.6  1993/12/08  23:42:06  yak
 * 	Expended copyright. CR 7534
 * 	[1993/12/08  23:01:53  yak]
 * 
 * Revision 1.1.2.5  1993/09/14  21:15:33  vania
 * 	*** empty log message ***
 * 	[1993/09/14  21:14:45  vania]
 * 
 * Revision 1.1.2.4  1993/09/13  12:09:29  vania
 * 	*** empty log message ***
 * 	[1993/09/13  12:08:36  vania]
 * 
 * Revision 1.1.2.3  1993/09/09  15:58:57  vania
 * 	added no proto
 * 	[1993/09/09  15:57:56  vania]
 * 
 * Revision 1.1.2.2  1993/08/26  21:52:45  vania
 * 	Initial revision
 * 	[1993/08/26  21:48:44  vania]
 * 
 * 	*** empty log message ***
 * 	[1993/08/26  21:24:13  vania]
 * 
 * $EndLog$
 */

/*
 * The Airport Drag And Drop Demo
 * The module that contains all drop site / receiver code
 */

#include "airport.h"
#include "dropsite.h"

/* ============================ prototypes  ======================= */

#ifdef _NO_PROTO
static void TrafficSignal();
static void SomethingLanding();
static void TaxiIn();
#else
static void TrafficSignal(Widget dropSite, XtPointer client_data,
			  XmDragProcCallbackStruct *call_data);
/* drag proc callback on airport. Something is flying over */

static void SomethingLanding(Widget dc, XtPointer client_data,
			     XmDropProcCallbackStruct *call_data);
/* drop proc callback on airport. Something is landing */

static void TaxiIn(Widget transfer,  Airport this,
		   Atom * selection, Atom * type, XtPointer value,
		   unsigned long *length, int *format);
/* Xt Selection Callback. Transfer flight number from source */
#endif

/* ===============================================================
 *   Expose callback. Redraw the airport
 */
#ifdef _NO_PROTO
void MakeAirportDropSite(area, this)
		Widget area; 
		Airport this;
#else
void MakeAirportDropSite(Widget area, Airport this)
#endif
{
   Arg args[10];
   int n = 0;
   
   XtSetArg(args[n], XmNimportTargets, &FlightAtom); n++;
   XtSetArg(args[n], XmNnumImportTargets, 1); n++;
   XtSetArg(args[n], XmNdragProc, TrafficSignal); n++;
   XtSetArg(args[n], XmNdropProc, SomethingLanding); n++;
   XtVaSetValues(area, XmNuserData, this, NULL);
   XmDropSiteRegister(area, args, n);
}

/* ===============================================================
 * Drag proc callback of the drop site.
 * The airport control is detecting/loosing radar echo on enter/leave
 * On motion events do animation of the tracks (flashing)
 */

#ifdef _NO_PROTO
static void TrafficSignal(dropSite, client_data, call_data)
		Widget dropSite;
		XtPointer client_data;
		XmDragProcCallbackStruct *call_data;
#else
static void TrafficSignal(Widget dropSite, XtPointer client_data,
			  XmDragProcCallbackStruct *call_data)
#endif
{
   Airport this;

   XtVaGetValues(dropSite, XmNuserData, &this, NULL);
   switch(call_data->reason) { 
      
   case XmCR_DROP_SITE_MOTION_MESSAGE:
      DrawAirport(this, XtWindow(dropSite),
		  AirportResources.drag_under_background,
		  AirportResources.drag_under_foreground);
      break;
      
   case XmCR_DROP_SITE_ENTER_MESSAGE:
      XmToggleButtonSetState(this->to.enter, True, True);
      break;

   case XmCR_DROP_SITE_LEAVE_MESSAGE: 
      XmToggleButtonSetState(this->to.leave, True, True);
      DrawAirport(this, XtWindow(dropSite),
		  AirportResources.track_background,
		  AirportResources.track_foreground);
      break;

   }
}

/* ===============================================================
 * Drop proc callback of the drop site. 
 * The airport knows that something is landing. What is it ?
 * If it is a plane then check there is a gate available.
 * 	If no gate available, reject the drop
 * If it is not a plane landing, reject the drop too.
 */

#ifdef _NO_PROTO
static void SomethingLanding(dropSite, client_data, call_data)
		Widget dropSite;
		XtPointer client_data;
		XmDropProcCallbackStruct *call_data;
#else
static void SomethingLanding(Widget dropSite, XtPointer client_data,
			     XmDropProcCallbackStruct *call_data)
#endif
{
#define NUMREC 2
   Arg args[16];
   int n;
   XmDropTransferEntryRec target_data[NUMREC];
   Airport this;
   unsigned char status = XmTRANSFER_FAILURE;
   int num_targets = 0;

   XtVaGetValues(dropSite, XmNuserData, &this, NULL);
   XmToggleButtonSetState(this->to.landing, True, True);
   n = 0;
   if (call_data->dropSiteStatus == XmVALID_DROP_SITE)
     {
	int gate;
	      
	for (gate = 0; gate < this->park.spot_count; gate ++) {
	   if (this->park.spots[gate].empty == True)
	     break;
	}
	if (gate == this->park.spot_count)
	  Paging(this, "Airport saturated, no landing");
	else {
	   status = XmTRANSFER_SUCCESS;
	   call_data->operation = XmDROP_MOVE;
	   num_targets = 1;
	   this->to.gate = gate;
	   target_data[0].client_data = (XtPointer) this;
	   target_data[0].target = FlightAtom;
	   XtSetArg(args[n], XmNtransferProc, TaxiIn);  n++;
	   XtSetArg(args[n], XmNdropTransfers, target_data);  n++;
	}
     }
   else {
      Paging(this, "Non Identified Flying Object is landing");
   }
   XtSetArg(args[n], XmNnumDropTransfers, num_targets);  n++;
   XtSetArg(args[n], XmNtransferStatus, status);  n++;
   XmDropTransferStart(call_data->dragContext, args, n);
#undef NUMREC
}

/* ===============================================================
 * The selection callback proc after drop transfer
 * Accept plane at free gate and signal arrival on paging.
 * If some error occured during the transfer, do plane crash
 */

#ifdef _NO_PROTO
static void TaxiIn(transfer, this, selection, type, value, length, format)
		Widget transfer;
		Airport this;
		Atom * selection;
		Atom * type;
		XtPointer value;
		unsigned long *length;
		int *format;
#else
static void TaxiIn(Widget transfer, Airport this,
		   Atom * selection, Atom * type, XtPointer value,
		   unsigned long *length, int *format)
#endif
{
   if ((*format == 32) && (*length == 1) && (*type == FlightAtom)) {
      /* 
       * received a 32 bit value that is the flight number
       */
      long flight_number;
      char msg[256];

      flight_number =  *((long *)value); 
      this->park.spots[this->to.gate].empty = False;
      sprintf(msg, "Flight %d arrived at gate %d",
	   flight_number, this->to.gate);
      Paging(this, msg);
      DrawAirport(this, XtWindow(this->airport),
		  AirportResources.track_foreground,
		  AirportResources.track_background);
      XmToggleButtonSetState(this->to.leave, True, True);
      XtFree(value);
  }
   else {
     Paging(this, "Too bad: plane has crashed in landing");
     XtVaSetValues(transfer,
		   XmNtransferStatus, XmTRANSFER_FAILURE,
		   XmNnumDropTransfers, 0,
		   NULL);
     return;
  }
}
