/*
 * COPYRIGHT NOTICE
 * Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 * ALL RIGHTS RESERVED (MOTIF).  See the file named COPYRIGHT.MOTIF
 * for the full copyright text.
 * 
 */
/*
 * HISTORY
 * $Log: actions.c,v $
 * Revision 1.1.2.5  1994/06/15  17:00:25  drand
 * 	Added ANSI prototypes
 * 	[1994/06/15  17:00:05  drand]
 *
 * Revision 1.1.2.4  1994/06/14  14:36:27  drand
 * 	Fix build problems
 * 	[1994/06/14  14:36:05  drand]
 * 
 * Revision 1.1.2.3  1994/06/10  21:55:14  drand
 * 	Much added, much fixed
 * 	[1994/06/10  21:54:53  drand]
 * 
 * Revision 1.1.2.2  1994/06/09  21:52:45  drand
 * 	Added configurable actions and icons
 * 	[1994/06/09  21:51:11  drand]
 * 
 * Revision 1.1.2.1  1994/06/02  15:35:30  marty
 * 	File moved from ./demos directory.
 * 	[1994/06/02  14:25:12  marty]
 * 
 * Revision 1.1.4.3  1994/04/20  18:34:01  drand
 * 	Fix bugs,  and update functionality
 * 	[1994/04/20  18:33:39  drand]
 * 
 * Revision 1.1.4.2  1994/04/06  19:06:26  drand
 * 	Changed interface to C base.  Added a little online help
 * 	[1994/04/06  19:05:52  drand]
 * 
 * Revision 1.1.4.1  1994/02/08  18:11:33  drand
 * 	Fix sizing of Container
 * 	[1994/02/08  16:54:04  drand]
 * 
 * Revision 1.1.2.2  1993/11/16  22:16:22  drand
 * 	Made some functional enhancements.  Added views
 * 	[1993/11/16  22:15:54  drand]
 * 
 * Revision 1.1.2.1  1993/10/25  20:18:58  drand
 * 	Navigation works
 * 	[1993/10/25  20:18:31  drand]
 * 
 * $EndLog$
 */

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <pwd.h>
#include <Xm/Xm.h>           /* Motif Toolkit */
#include <Mrm/MrmPublic.h>   /* Mrm */
#include <Xm/SelectioB.h>
#include <Xm/TextF.h>
#include "filemanager.h"

void 
#ifdef _NO_PROTO
gotoCB( widget, i1, cb)
     Widget widget;
     XtPointer i1, cb;
#else
gotoCB(Widget widget, XtPointer i1, XtPointer cb)
#endif /* _NO_PROTO */
{
  Widget text;
  char *str;

  text = XtNameToWidget(widget, "*Text");

  str = XmTextFieldGetString(text);

  readdirCB(widget, str, NULL);

  XtFree(str);
}

void 
#ifdef _NO_PROTO
manageCB( widget, w_to_manage, callback_data)
     Widget widget;
     Widget w_to_manage;
     XtPointer callback_data;
#else
manageCB(Widget widget, Widget w_to_manage, XtPointer callback_data)
#endif /* _NO_PROTO */
{
  if (w_to_manage != (Widget) NULL)
    XtManageChild(w_to_manage);
}

void 
#ifdef _NO_PROTO
viewCB( widget, type, callback_data)
     Widget widget;
     char   *type;
     XtPointer callback_data;
#else
viewCB(Widget widget, char   *type, XtPointer callback_data)
#endif /* _NO_PROTO */
{
  XtEnum size, spatial;

  if (strcmp(type, "large") == 0) {
    size = XmLARGE_ICON;
    spatial = XmSPATIAL;
  } else if (strcmp(type, "small") == 0) {
    size = XmSMALL_ICON;
    spatial = XmSPATIAL;
  } else if (strcmp(type, "detail") == 0) {
    size = XmSMALL_ICON;
    spatial = XmDETAIL;
  }

  XtVaSetValues(fileviewer,
		XmNlayoutType, spatial,
		XmNentryViewType, size,
		0, 0);

  fixViewerSize(XtParent(fileviewer), NULL, NULL);

  XmContainerRelayout(fileviewer);
}

void 
#ifdef _NO_PROTO
quitCB( widget, tag, callback_data )
     Widget	widget;
     char       *tag;
     XmAnyCallbackStruct *callback_data;
#else
quitCB(Widget widget, char *tag, XmAnyCallbackStruct *callback_data)
#endif /* _NO_PROTO */
{
  exit(0);
}

char* 
#ifdef _NO_PROTO
fullpath(filename)
     char *filename;
#else
fullpath(char *filename)
#endif /* _NO_PROTO */
{
  char buf[1024];
  char *rstr;

  /* This is for UNIX,  write something else for a non-UNIX OS */
  if (strcmp(currentdir, "/") == 0)
    sprintf(buf, "/%s", filename);
  else if (filename[0] == '/')
    sprintf(buf, "%s%s", currentdir, filename);
  else
    sprintf(buf, "%s/%s", currentdir, filename);

  rstr = XtNewString(buf);

  return(rstr);
}


void 
#ifdef _NO_PROTO
showHiddenCB(widget, ignore, callback_data)
     Widget	widget;
     XtPointer	ignore;
     XmToggleButtonCallbackStruct *callback_data;
#else
showHiddenCB(Widget widget, XtPointer ignore,
	     XmToggleButtonCallbackStruct *callback_data)
#endif /* _NO_PROTO */
{
  showHidden = callback_data -> set;

  read_directory((Widget) NULL, ".");
}

void 
#ifdef _NO_PROTO
newFolder(widget, ignore, ignore2)
     Widget	widget;
     XtPointer	ignore, ignore2;
#else
newFolder(Widget widget, XtPointer ignore, XtPointer ignore2)
#endif /* _NO_PROTO */
{
  char buf[256];
  int i = 0;
  int status;
  Boolean done = False;
  struct stat statbuf;

  while(! done) {
    sprintf(buf, "%s/unnamed%d", currentdir, i);
    status = stat(buf, &statbuf);
    if (status != 0) {
      if (errno == ENOENT)
	done = True;
      else
	return; /* Bad error */
    }
    if (i >= 100) return;
    i++;
  }

  mkdir(buf, 0755);

}
     
void 
#ifdef _NO_PROTO
deleteItem(widget, ignore, ignore2)
     Widget	widget;
     XtPointer	ignore, ignore2;
#else
deleteItem(Widget widget, XtPointer ignore, XtPointer ignore2)
#endif /* _NO_PROTO */
{
  WidgetList selected;
  Cardinal count;
  int i;

  /* First get list of selected items. */
  XtVaGetValues(fileviewer, 
		XmNselectedObjects, &selected,
		XmNselectedObjectCount, &count,
		NULL, NULL);

  if (count <= 0) return;
 
  for(i = 0; i < count; i++) {
    char buf[256];
    sprintf(buf, deleteCommand, getPathFromIcon(selected[i]));
    system(buf);
  }

  XtVaSetValues(fileviewer, XmNselectedObjectCount, 0, NULL, NULL);
}




