/* 
 *  @OSF_COPYRIGHT@
 *  COPYRIGHT NOTICE
 *  Copyright (c) 1990, 1991, 1992, 1993 Open Software Foundation, Inc.
 *  ALL RIGHTS RESERVED (MOTIF). See the file named COPYRIGHT.MOTIF for
 *  the full copyright text.
 */ 
/* 
 * HISTORY
 * $Log: transfer.c,v $
 * Revision 1.1.2.2  1994/07/07  17:25:34  drand
 * 	Change app class to follow naming convention
 * 	[1994/07/07  17:09:13  drand]
 *
 * Revision 1.1.2.1  1994/06/02  15:38:49  marty
 * 	File moved from ./demos directory.
 * 	[1994/06/02  14:28:34  marty]
 * 
 * Revision 1.1.9.2  1994/05/18  18:31:02  drand
 * 	Added _NO_PROTO/PROTO headers
 * 	[1994/05/18  18:30:49  drand]
 * 
 * Revision 1.1.9.1  1994/05/13  21:59:58  drand
 * 	Added functional prototypes and changed to use menu creation funcs
 * 	[1994/05/13  21:59:33  drand]
 * 
 * Revision 1.1.7.5  1994/04/14  17:25:07  drand
 * 	Fix build warning
 * 	[1994/04/14  17:24:09  drand]
 * 
 * Revision 1.1.7.4  1994/04/11  15:26:41  drand
 * 	Updated to be able to mark targets deferred for Clipboard
 * 	[1994/04/11  15:26:22  drand]
 * 
 * Revision 1.1.7.3  1994/04/05  19:29:02  drand
 * 	Converted from UIL to C interface
 * 	[1994/04/05  19:26:48  drand]
 * 
 * Revision 1.1.7.2  1994/03/21  23:01:24  drand
 * 	Added support for SET and REMOVE.
 * 	[1994/03/21  23:01:07  drand]
 * 
 * Revision 1.1.7.1  1994/01/31  18:15:54  drand
 * 	Add testing for NamedAtom
 * 	[1994/01/31  18:15:39  drand]
 * 
 * Revision 1.1.5.7  1993/10/14  20:48:27  drand
 * 	Now uses an option menu to set the format to view the target
 * 	data.  Use CSText to display data.
 * 	[1993/10/14  20:43:46  drand]
 * 
 * Revision 1.1.5.6  1993/09/28  20:45:27  drand
 * 	Added handling for the ownership toggles.  Added dragging from the
 * 	target drawing area widget.
 * 	[1993/09/28  20:43:52  drand]
 * 
 * Revision 1.1.5.5  1993/09/22  20:33:32  drand
 * 	Now catch XErrors and put the text in the textfield. Now optionally
 * 	use TextField.
 * 	[1993/09/22  20:31:43  drand]
 * 
 * Revision 1.1.5.4  1993/09/17  20:54:57  drand
 * 	Checkpoint #2
 * 	[1993/09/17  20:53:53  drand]
 * 
 * Revision 1.1.5.3  1993/09/17  14:28:02  drand
 * 	Checkpoint for demo.
 * 	[1993/09/17  14:26:19  drand]
 * 
 * Revision 1.1.5.2  1993/09/10  20:44:19  drand
 * 	New versions.
 * 	[1993/09/10  20:39:35  drand]
 * 
 * $EndLog$
 */ 

#ifdef REV_INFO
#ifndef lint
static char rcsid[] = "$RCSfile: transfer.c,v $ $Revision: 1.1.2.2 $ $Date: 1994/07/07 17:25:34 $"
#endif
#endif

#include <stdio.h>
#include <Xm/Xm.h>           /* Motif Toolkit */
#include "transfer.h"

#ifdef _NO_PROTO
void quitCB();
void setMode();
void cut();
void copy();
void paste();
void pasteMove();

static void NOP();
static int ErrorHandler();
#else
void quitCB(Widget widget, char *tag, XmAnyCallbackStruct *callback_data);
void setMode(Widget widget, int param, XmAnyCallbackStruct *callback_data);
void cut(Widget widget, char *ignore, XmAnyCallbackStruct *callback_data);
void copy(Widget widget, char *ignore, XmAnyCallbackStruct *callback_data);
void paste(Widget widget, char *ignore, XmAnyCallbackStruct *callback_data);
void pasteMove(Widget widget, char *ignore, 
	       XmAnyCallbackStruct *callback_data);

static void NOP(Widget widget, XEvent *event, 
		String *params, Cardinal *num_params);
static int ErrorHandler(Display *display, XErrorEvent *event);
#endif /* _NO_PROTO */

/* String to use for Clipboard tagging */
XmString generic_label;
Display  	*display;
XtAppContext  	app_context;

/* This tracks which transfer selection we're using.  Drag and drop isn't
   listed since it is dynamic,  we don't take ownership of DnD */
TransferMode Tmode = Mprimary;

/* We need a NOP action */
static XtActionsRec actions[] = {
  { "NOP",		NOP},
};

/*************************************************************************
 * A list of widgets we need to be able to find outside the main
 * program.
 *
 * toplevel	is the ApplicationShell
 * mainw	is the child of toplevel,  an XmMainWindow
 * target	is the drawing area used for drag and drop
 * list		is the list of targets displayed
 * text1	contains the value of the selected target
 * text2	contains the name of the selected or new target
 * typeOM	is the option menu which displays type information for the
 *  		selected or new target
 * format_om    Format option menu
 * format_s	String format button
 * format_l	Locale format button
 * format_t	CompoundText format button
 * format_cs	CompoundString format button
 * format_id	Integer format button (decimal)
 * format_hd	Integer format button (hex)
 *************************************************************************/

Widget toplevel, mainW, editBox;

/*
 *  Main program
 */

int 
#ifdef _NO_PROTO
main(argc, argv)
     int argc;
     String *argv;
#else
main(int argc, String *argv)
#endif /* _NO_PROTO */
{
    Arg	args[10];
    int n;

    toplevel = XtVaAppInitialize(&app_context, "XmdTransfer", NULL, 0, 
				 &argc, argv, NULL, 0);

    /* Add the NOP actions */
    XtAppAddActions(app_context, actions, XtNumber(actions));

    /* Create edit dialog */
    editBox = create_editbox_widget("edit_box", toplevel);

    /* Initialize the interface */
    mainW = create_mainWindow_widget("main", toplevel);

    /* Create a default label string for the clipboard */
    generic_label = XmStringCreateLocalized("Generic Label");

    /* Register the drawing area as a drop site */
    XmeDropSink(target, NULL, 0);

    XtManageChild(mainW);

    /* Install our special error handler */
    XSetErrorHandler((XErrorHandler) ErrorHandler);

    /* Add the UTM callbacks on the drawing area */

    XtAddCallback(target, XmNdestinationCallback, 
		  (XtCallbackProc) targetDestinationCallback, NULL);
    XtAddCallback(target, XmNconvertCallback, 
		  (XtCallbackProc) targetConvertCallback, NULL);

    /* Add other callbacks on the drawing area */
    XtAddCallback(target, XmNinputCallback,
		  (XtCallbackProc) inputCallback, NULL);
    
    XtRealizeWidget(toplevel);

    XtAppMainLoop(app_context);
}

/***************************************************
 * function setMode				  
 * Set the mode to PRIMARY, SECONDARY or CLIPBOARD
 ***************************************************/
void 
#ifdef _NO_PROTO
setMode(widget, param, callback_data)
     Widget widget;
     int param;
     XmAnyCallbackStruct *callback_data;
#else
setMode(Widget widget, int param, XmAnyCallbackStruct *callback_data)
#endif /* _NO_PROTO */
{
  Tmode = param;
}

void 
#ifdef _NO_PROTO
quitCB(widget, tag, callback_data)
     Widget widget;
     char *tag;
     XmAnyCallbackStruct *callback_data;
#else
quitCB(Widget widget, char *tag, XmAnyCallbackStruct *callback_data)
#endif /* _NO_PROTO */
{
  exit(0);
}

/******************************************************************
 * cut, copy and paste perform the apropriate actions
 * depending on which mode we're in.  They use the widget writer's
 * API for this.  Normally application developers won't need to
 * call these,  but this is a pseudo widget writing case as we're
 * adding interchange behavior to drawing area.  This is an example
 * of where the widget writing API should be used by app. developers
 ******************************************************************/

void 
#ifdef _NO_PROTO
cut(widget, ignore, callback_data)
     Widget widget;
     char *ignore; 
     XmAnyCallbackStruct *callback_data;
#else
cut(Widget widget, char *ignore, XmAnyCallbackStruct *callback_data)
#endif /* _NO_PROTO */
{
  if (Tmode == Mprimary) {
    XmePrimarySource(target, 0);
    XmToggleButtonSetState(op, True, False);
  } else if (Tmode == Mnamed) {
    XmeNamedSource(target, XInternAtom(XtDisplay(target), _NAMED, False), 0);
    XmToggleButtonSetState(onamed, True, False);
  } else if (Tmode == Msecondary) {
    XmeSecondarySource(target, 0);
    XmeSecondaryTransfer(target, 
			 XInternAtom(XtDisplay(target), 
				     "COMPOUND_TEXT", False),
			 XmMOVE, 0);
  } else if (Tmode == Mclipboard)
    XmeClipboardSource(target, XmMOVE, 0, generic_label);
}


void 
#ifdef _NO_PROTO
copy(widget, ignore, callback_data)
     Widget widget;
     char *ignore;
     XmAnyCallbackStruct *callback_data;
#else
copy(Widget widget, char *ignore, XmAnyCallbackStruct *callback_data)
#endif /* _NO_PROTO */
{
  if (Tmode == Mprimary) {
    XmePrimarySource(target, 0);
    XmToggleButtonSetState(op, True, False); 
  } else if (Tmode == Mnamed) {
    XmeNamedSource(target, XInternAtom(XtDisplay(target), _NAMED, False), 0);
    XmToggleButtonSetState(onamed, True, False);
  } else if (Tmode == Msecondary) {
    XmeSecondarySource(target, 0);
    XmeSecondaryTransfer(target, 
			 XInternAtom(XtDisplay(target), 
				     "COMPOUND_TEXT", False),
			 XmCOPY, 0);
  } else if (Tmode == Mclipboard)
    XmeClipboardSource(target, XmCOPY, 0, generic_label);
}

void 
#ifdef _NO_PROTO
paste(widget, ignore, callback_data)
     Widget widget; 
     char *ignore;
     XmAnyCallbackStruct *callback_data;
#else
paste(Widget widget, char *ignore, XmAnyCallbackStruct *callback_data)
#endif /* _NO_PROTO */
{
  if (Tmode == Mprimary) {
    XmePrimarySink(target, XmCOPY, NULL, 0);
  } else if (Tmode == Mnamed) {
    XmeNamedSink(target, XInternAtom(XtDisplay(target), _NAMED, False),
		 XmCOPY, NULL, 0);
  } else if (Tmode == Msecondary) {
    XmeSecondarySink(target, 0);
    XmToggleButtonSetState(omd, True, False);
  } else if (Tmode == Mclipboard)
    XmeClipboardSink(target, XmCOPY, 0);
}

void 
#ifdef _NO_PROTO
pasteMove(widget, ignore, callback_data)
     Widget widget; 
     char *ignore;
     XmAnyCallbackStruct *callback_data;
#else
pasteMove(Widget widget, char *ignore, XmAnyCallbackStruct *callback_data)
#endif /* _NO_PROTO */
{
  if (Tmode == Mprimary) {
    XmePrimarySink(target, XmMOVE, 0, 0);
  } else if (Tmode == Mnamed) {
    XmeNamedSink(target, XInternAtom(XtDisplay(target), _NAMED, False),
		 XmMOVE, NULL, 0);
  } else if (Tmode == Msecondary) {
    XmeSecondarySink(target, 0);
    XmToggleButtonSetState(omd, True, False);
  }
  else if (Tmode == Mclipboard)
    XmeClipboardSink(target, XmMOVE, 0);
}

/* Error handler for X protocol errors.  Display error text in text1,2 */

static int 
#ifdef _NO_PROTO
ErrorHandler(display, event)
     Display *display;
     XErrorEvent *event;
#else
ErrorHandler(Display *display, XErrorEvent *event)
#endif /* _NO_PROTO */
{
  char buffer[80];
  char errortext[100];
  XmString tmp;

  XGetErrorText(display, event -> error_code, errortext, 100);

  tmp = XmStringCreateLocalized(errortext);
  XmCSTextSetString(text2, tmp);
  XmStringFree(tmp);

  sprintf(buffer, "XID %d serial %d major %d minor %d",
	  event -> resourceid,
	  event -> request_code,
	  event -> minor_code);

  tmp = XmStringCreateLocalized(buffer);
  XmCSTextSetString(text1, tmp);
  XmStringFree(tmp);

  return 0;
}

void 
#ifdef _NO_PROTO
inputCallback(w, ignore, cbstruct)
     Widget w;
     XtPointer ignore;
     XmDrawingAreaCallbackStruct *cbstruct;
#else
inputCallback(Widget w, XtPointer ignore, XmDrawingAreaCallbackStruct *cbstruct)
#endif /* _NO_PROTO */
{
  XEvent *event = cbstruct -> event;

  if (event -> type == ButtonPress &&
      event -> xbutton.button == 2) {
    XmeDragSource(w, NULL, event, NULL, 0);
  }
}

static void 
#ifdef _NO_PROTO
NOP(widget, event, params, num_params)
     Widget widget;
     XEvent *event;
     String *params;
     Cardinal *num_params;
#else
NOP(Widget widget, XEvent *event, String *params, Cardinal *num_params)
#endif /* _NO_PROTO */
{
  /* We do nothing in the action. */
}

